function CBaseEntity_GetEntityColor()
{
	local color = GetPropInt(this, "m_clrRender");
    local clr = {}
    clr.r <- color & 0xFF;
    clr.g <- (color >> 8) & 0xFF;
    clr.b <- (color >> 16) & 0xFF;
    clr.a <- (color >> 24) & 0xFF;
    return clr;
}

::GetGroundEntity <- function(entity)
{
	return GetPropEntity(entity, "m_hGroundEntity")
}

function CBaseEntity_GetAbsOrigin()
{
	return GetPropVector(this, "m_vecAbsOrigin");
}

function CBaseEntity_IsWorld()
{
	if (this != null)
	{
		if (this.GetClassname() == "worldspawn")
		{
			return true
		}
		else
		{
			return false
		}
	}
	else
	{
		return false
	}
}

function CBaseEntity_GetWaterLevel()
{
	return GetPropInt(this, "m_nWaterLevel");
}

function CBaseEntity_PhysicsCheckWater()
{
	return this.GetWaterLevel() > WL_Feet;
}

function CBaseEntity_GetElasticity()
{
	return GetPropFloat(this, "m_flElasticity")
}

::GetActualGravity <- function( pEnt )
{
	local ent_gravity = pEnt.GetGravity(); //float
	if ( ent_gravity == 0.0 )
	{
		ent_gravity = 1.0;
	}

	return ent_gravity * Convars.GetFloat("sv_gravity");
}

function CBaseEntity_SetLocalAngularVelocity( vecAngVelocity )
{
	if (GetPropVector(this, "m_vecAngVelocity") != vecAngVelocity)
	{
		SetPropVector(this, "m_vecAngVelocity", vecAngVelocity)
	}
}

::SetGroundEntity <- function(entity, ground)
{
	if ( GetGroundEntity(entity) == ground )
		return;

	SetPropEntity(entity, "m_hGroundEntity", ground)
}

function CBaseEntity_IsEffectActive(effect)
{
	if ((GetPropInt(this, "m_fEffects") & effect) == effect)
	{
		return true
	}
	else
	{
    	return false;
	}
}

function CBaseEntity_SetBaseVelocity( v )
{
    SetPropVector(this, "m_vecBaseVelocity", v)
}

function CBaseEntity_IsStandable()
{
	if (this.GetSolidFlags() & FSOLID_NOT_STANDABLE)
		return false;

	if ( this.GetSolid() == SOLID_BSP || this.GetSolid() == SOLID_VPHYSICS || this.GetSolid() == SOLID_BBOX )
		return true;

	return this.IsBSPModel( );
}

function CBaseEntity_SetElasticity(float)
{
	SetPropFloat(this, "m_flElasticity", float)
}

function CBaseEntity_SetEntityColor(r, g, b, a)
{
    local color = (r) | (g << 8) | (b << 16) | (a << 24);
    SetPropInt(this, "m_clrRender", color);
}

function CBaseEntity_IsBSPModel()
{
	local entity = this

	if ( entity.GetSolid() == SOLID_BSP )
		return true;

	if ( entity.GetSolid() == SOLID_VPHYSICS)
		return true;

	return false;
}

function CBaseEntity_RemoveEffects(effect)
{
	local m_fEffects = GetPropInt(this, "m_fEffects")
	SetPropInt(this, "m_fEffects", m_fEffects - effect)
}

function CBaseEntity_AddEffects(effect)
{
	local m_fEffects = GetPropInt(this, "m_fEffects")
	if (!((m_fEffects & effect) == effect))
	{
		SetPropInt(this, "m_fEffects", m_fEffects + effect)
	}
}

function CBaseEntity_FollowEntity(pBaseEntity, bBoneMerge)
{
	local pEntityToMerge = this

	if (pEntityToMerge)
	{
		EntFireByHandle(pEntityToMerge, "SetParent", "!activator", 0, pBaseEntity, pBaseEntity);

		pEntityToMerge.SetMoveType(0, 0);

		if ( bBoneMerge )
		{
			SetPropInt(pEntityToMerge,"m_fEffects", 129);
		}

		SetPropInt(pEntityToMerge,"m_CollisionGroup", 0);
	}
}

function CBaseEntity_GetLocalAngularVelocity()
{
	return GetPropVector(this, "m_vecAngVelocity");
}

local entity_classes = [];

foreach (key, value in getroottable())
{
    if (typeof(value) == "class" && "AddEFlags" in value)
        entity_classes.append(value);
}

foreach (key, value in this)
{
    if (typeof(value) == "function" && startswith(key, "CBaseEntity_"))
    {
        local func_name = key.slice(12);
        foreach (entity_class in entity_classes)
            entity_class[func_name] <- value;
        delete this[key];
    }
}