//-----------------------------------------------------------------------------
//	Purpose: Gives a weapon to the player.
//			 This script's bread 'n' butter.
//-----------------------------------------------------------------------------

::CTFPlayer.GiveWeapon <- function(weapon,ItemDefinitionIndex_Override=null,gw_props=Defaults,PostWepFix_Override=null,stats_function_Override=null)
{
// Starts with GetWeaponTable(weapon).
// This searches for the weapon in the database, and returns an assembled table with all the properties we need to make this weapon!
	local table;
// if we just put a classname in "weapon", pick the first "if" statement. Else, use the second statement.
	local weapon_String_Classname_Type = false;
	local weapon_tolower = type ( weapon ) == "string" ? weapon.tolower() : "";
	if ( weapon_tolower && startswith(weapon_tolower, "tf_wea") ) {
		if ( weapon_tolower.len() < 13 ) return GTFW.DevPrint(1,"GiveWeapon", "Function failed. Incomplete string \x22"+weapon+"\x22. Returning null.");
		
		weapon_String_Classname_Type = startswith(weapon_tolower, "tf_weapon_") ? "tf_weapon_" : "tf_wearable_";
		weapon_tolower = startswith(weapon_tolower, "tf_weapon_") ? weapon_tolower.slice(10) : weapon_tolower.slice(12);
		table = GetWeaponTableNoPlayer(weapon_tolower);
	}
	else { table = this.GetWeaponTable(weapon); }
	
// Error if there is no valid item found
	if ( table == null ) {
		GTFW.DevPrint(1,"GiveWeapon", "Function failed at finding item \x22"+weapon+"\x22. Returning null.");
		return null;
	}
	
// Fixes up the weapon if override present
	local PostWepFix = table.PostWepFix;
	if ( type( PostWepFix_Override ) == "function" ) {
		PostWepFix = PostWepFix_Override(table, this);
	}
	
// Define variables
	local player = this;
	
// Establish all variables from the table inside this script
	local WepC				= table.classname;
	local PlayerTFClass		= table.tf_class;
	local ItemID			= table.itemID;
	local Slot				= table.slot;
	local AmmoType			= table.ammoType;
	local ClipSize			= table.clipSize;
	local MaxClipSize		= table.clipSizeMax;
	local AmmoReserve		= table.reserve;
	local MaxAmmoReserve	= table.reserveMax;
	local Extra_Wearable	= table.wearable;
	local Extra_Wearable_VM	= table.wearable_vm;

	local AnnounceQuality	= table.announce_quality;
	local AnnouncePrefix	= table.announce_prefix;
	local AnnounceHasString	= table.announce_has_string;
	
	local worldModel		= table.worldModel;
	local viewModel		 	= table.viewModel;
	local stats_function 	= table.func;
	local classArms		 	= table.classArms;
	local classAnimsModel	= table.classAnimsModel;
	
	if ( ItemDefinitionIndex_Override ) {
		local overrides	= GetWeaponTableNoPlayer(ItemDefinitionIndex_Override)
		ItemID			= overrides.itemID;
		PlayerTFClass	= overrides.tf_class;
	}

// Purpose: Create weapon by classname
// If you wanted to create a weapon by classname + Item Definition Index, this is the way.
	if ( weapon_String_Classname_Type ) { // set to `true` at beginning of script
		WepC = weapon_String_Classname_Type + weapon_tolower;	//adds "tf_weapon_" or "tf_wearable_" back
	}

	
// Purpose: Extra properties for the weapon.
//	It utilizes a set of bits. This works exactly how the most entities with `effects` keyvalue can hold so many options inside one number!
	if ( type ( gw_props ) != "integer" ) { gw_props = Defaults; }
	else { gw_props = gw_props | table.gw_props; }

	local bit_DeleteAndReplace	= gw_props & DeleteAndReplace ? true : false;
	local bit_KeepIDX			= gw_props & KeepIDX ? true : false;
	local bit_AutoSwitch		= gw_props & AutoSwitch ? true : false;
	local bit_WipeAttributes	= gw_props & WipeAttributes ? true : false;
	local bit_ForceCustom		= gw_props & ForceCustom ? true : false;
	local bit_ForceNotCustom	= gw_props & ForceNotCustom ? true : false;
	local bit_AnnounceToChat	= gw_props & AnnounceToChat ? true : false;
	local bit_SaveWeapon		= gw_props & Save ? true : false;
	local bit_AutoRegister		= gw_props & AutoRegister ? true : false;

	
// Purpose: Weapon Creation
// It finds a slot to replace, deletes that weapon in slot, then creates either a passive item (i.e. booties, etc) or an active weapon.

	local GivenWeapon;

// This thing is might be huge, but the bulk of it is finding and deleting passive items. (also, active ones)
	if (  bit_KeepIDX == false ) {
		//If ReplaceWeapon is set in bitmasks, it finds the weapon in the slot the new weapon would be in, then deletes it.
		if ( bit_DeleteAndReplace ) {
			for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++) {
					
				local wep = GetPropEntityArray(player, "m_hMyWeapons", i);
				
				if ( wep == null ) continue
			//Weapons that are in an unintended slot always return their original slot index. It would bug at this point if we only checked using GetSlot().
			//So all scopes/info for weapons are created on post_inventory_application
					if ( ( "slot" in wep.GetScriptScope() && wep.GetScriptScope().slot == Slot ) ) {
				//	  || wep.GetSlot() == Slot ) {
						SetPropEntityArray(player, "m_hMyWeapons", null, i);
						KillWepAll(wep);
						break
					}
			}
			
	// Finds passive weapons like Gunboats, Targe, etc.
			local item = player.GetPassiveWeaponBySlot(Slot);
			if ( item ) {

				if ( item.GetClassname() == "tf_wearable_demoshield" )
					SetPropInt(player, "m_Shared.m_bShieldEquipped", 0);
				
				KillWepAll(item);
			}
			GTFW.DevPrint(0,"GiveWeapon", "Removed item from slot "+Slot+"!");
		}
		
	// Purpose: This creates the new items
	// Passive items: If the classname matches any of these under SearchPassiveItems (list in tables.nut), it creates a wearable. The "CreateCustomWearable()" function does the rest.
	// Active items: else, it creates the new active weapon using the "AddWeapon" function.
	
	// Purpose: Creates a passive item	
		if ( SearchPassiveItems.find(WepC) ) {
			GivenWeapon = player.CreateCustomWearable(worldModel, ItemID);
			
			//Demoshield fix
			if ( WepC == "tf_wearable_demoshield" )
				SetPropInt(player, "m_Shared.m_bShieldEquipped", 1);
		}
		else {
	// Purpose: Creates an active weapon
			for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++) {
				
				local wep = GetPropEntityArray(player, "m_hMyWeapons", i);
				
				if ( wep ) continue
					GivenWeapon = player.AddWeapon(WepC, ItemID, i, Slot);
					break;
			}
		}
	}
	
// If "bit_keepIDX" set to true, it will only change the item definition index of the weapon.
	else {
		GivenWeapon = player.ReturnWeaponBySlot(Slot);
		if ( ItemID >= 0 ) {
			SetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", ItemID );
		}
		else {
			SetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", GetItemID(GivenWeapon) );
		}
		GTFW.DevPrint(0,"GiveWeapon", "bit_keepIDX = true. \x22"+weapon+"\x22 updated to IDX "+GetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")+"!");
	}
	
// Purpose: seems to throw an error only if it didn't create a weapon,
// that can only happen if weps in `m_hMyWeapons` exceeds GLOBAL_WEAPON_COUNT
	if ( GivenWeapon == null ) {
		GTFW.DevPrint(1,"GiveWeapon", "Function failed at giving weapon \x22"+weapon+"\x22. Returning null.");
		return null;
	}
	
// If our weapon has a custom model or custom arms, makes it custom.

	local main_viewmodel = GetPropEntity(player, "m_hViewModel");
	main_viewmodel.ValidateScriptScope()
	local VM_Scope = main_viewmodel.GetScriptScope();
	
	if ( !bit_ForceNotCustom
	 && (ItemID > 65535 )
	 || worldModel || viewModel || classArms
	 || bit_ForceCustom
	 || ItemDefinitionIndex_Override
	 || ( player.GetPlayerClass() != PlayerTFClass && PlayerTFClass != 0 ) )
	{
		IS_CUSTOM(GivenWeapon, true);
	}
	else IS_CUSTOM(GivenWeapon, false)
	
// Modifies ammo if ammo types don't match
	if ( HasProp(GivenWeapon, "LocalWeaponData.m_iPrimaryAmmoType") && AmmoType != GivenWeapon.GetPrimaryAmmoType() 
	  && GivenWeapon.GetPrimaryAmmoType() >= 0 ) {
		SetPropInt(GivenWeapon, "LocalWeaponData.m_iPrimaryAmmoType", AmmoType);
	}
	
// Purpose: Custom stats function
// Forced overrides parameters...
	if ( stats_function_Override ) {
		stats_function = stats_function_Override;
	}
// Runs custom weapon function if it exists
	if ( type( stats_function ) == "function" ) {
		stats_function(GivenWeapon, player);
		GivenWeapon.ReapplyProvision();
	}
	
// If our weapon does not use original stats, clear them.
	if ( bit_WipeAttributes ) {
		SetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", 1);	// Removes all pre-existing stats of the weapon.
	}
	
// Purpose: Extra wearable ( if applicable )
	if ( Extra_Wearable ) {
		player.CreateCustomWearable(Extra_Wearable, 65535, GivenWeapon);
	}
	
// Purpose: Extra wearable for viewmodels ( if applicable )
	if ( Extra_Wearable_VM ) {
		player.CreateCustomWearableViewModel( Extra_Wearable_VM, GivenWeapon);
	}

// Purpose: For updating weapon models
// for UpdateCustomWeapon())
	if ( worldModel == "" && viewModel == "" ) {
		local model = GetItemModelName(ItemID);
		worldModel = model;
		viewModel = model;
	}
	
	if ( worldModel == "" && viewModel ) {
		worldModel = viewModel;
	}
	else if ( worldModel && viewModel == "" ) {
		viewModel = worldModel;
	}
	
// Write GTFW table to weapon scope
	GivenWeapon.ValidateScriptScope();
	local wepScope = GivenWeapon.GetScriptScope();

	wepScope.weapon			<-	GivenWeapon;
	wepScope.tf_class		<-	PlayerTFClass;
	wepScope.slot			<-	Slot;
	wepScope.classname		<-	WepC;
	wepScope.itemID			<-	ItemID;
	wepScope.name			<-	table.name;
	wepScope.ammoType		<-	AmmoType;
	wepScope.clipSize		<-	ClipSize;
	wepScope.clipSizeMax	<-	MaxClipSize;
	wepScope.reserve		<-	AmmoReserve;
	wepScope.reserveMax		<-	MaxAmmoReserve;
	wepScope.worldModel		<-	worldModel;
	wepScope.viewModel		<-	viewModel;
	wepScope.wearable		<-	Extra_Wearable;
	wepScope.wearable_vm	<-	Extra_Wearable_VM;
	wepScope.gw_props		<-	gw_props;
	wepScope.func			<-	stats_function;
	wepScope.classArms		<-	classArms;
	wepScope.classAnimsModel		<-	classAnimsModel;
	wepScope.PostWepFix		<-	PostWepFix;
	//wepScope.IsCustom
	
	wepScope.announce_quality		<-	AnnounceQuality;
	wepScope.announce_prefix		<-	AnnouncePrefix;
	wepScope.announce_has_string	<-	AnnounceHasString;

// This tells the game it came here and only here
	wepScope.from_GiveWeapon <- true;
	
// Purpose: Finalize weapon.
// Adds AmmoFix wearable (to fix ammo), updates class arms/viewmodels, updates world weapon models, and adds viewmodel's Think script
	GTFW.UpdateAmmoFix(player, GivenWeapon,PlayerTFClass,AmmoType,ClipSize,MaxClipSize,AmmoReserve,MaxAmmoReserve);
	GTFW.UpdateArms(player, GivenWeapon);
	GTFW.UpdateCustomWeapon(player, GivenWeapon, worldModel, viewModel);
	local playerClassModel = GTFW.UpdateClassAnimations(player, GivenWeapon)
	GTFW.AddThinkToViewModel(player, playerClassModel);
	
// Purpose: Switches to another weapon if active one was deleted
	if ( bit_AutoSwitch )
		player.SwitchToBest(GivenWeapon);
	
	
// Purpose: Announces weapon in text chat that it has been found
	if ( bit_AnnounceToChat ) player.AnnounceWeapon(GetItemID(GivenWeapon),AnnounceQuality,AnnouncePrefix,AnnounceHasString);
		
// Purpose: Saves weapon so resupply or dying won't remove the weapon
	if ( bit_SaveWeapon ) player.SaveLoadout(weapon);
	
// Purpose: Automatically registers your given weapon as custom
// *Only use when you need it.* There can only be 63 custom weapons using the same item definition index.
	if ( bit_AutoRegister )
		RegisterCustomWeapon(UniqueString(), weapon, PostWepFix_Override,stats_function_Override);
	

// Purpose: Echoes to console who received what weapon
	if ( type( weapon ) == "integer" )
		weapon = table.name;
	GTFW.DevPrint(0,"GiveWeapon", "Gave "+GTFW.GetNetName(player)+" item \x22"+weapon+"\x22!");

	return GivenWeapon
}


//-----------------------------------------------------------------------------
//Base for making weapons. Base weapon netprops compiled by ficool2.
//-----------------------------------------------------------------------------
::CTFPlayer.AddWeapon <- function(classname, itemindex, slot, slot_in_loadout=null)
{
	GTFW.DevPrint(0,"AddWeapon",format("%s %i %i",classname, itemindex, slot))
	
	local ply = this;
	local gtfw_cleanup_wep;
	
	local weapon = SpawnEntityFromTable(classname, {
		origin = ply.GetOrigin(),
		angles = ply.GetAbsAngles(),
		effects = 129,
	});
	SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemindex);
	SetPropBool(weapon, "m_bValidatedAttachedEntity", true);
	SetPropBool(weapon, "m_AttributeManager.m_Item.m_bInitialized", true);
	SetPropInt(weapon, "m_AttributeManager.m_iReapplyProvisionParity", 1);
	weapon.SetTeam(ply.GetTeam());

	if ( classname != "tf_weapon_builder"
	  && classname != "tf_weapon_sapper"
	  && slot_in_loadout != weapon.GetSlot() ) {
		weapon.Kill();
		local dummy = "tf_weapon_";

		switch(slot_in_loadout)
		{
		case 0:
			dummy += "rocketlauncher";
			break;
		case 1:
			dummy += "smg";
			break;
		case 2:
			dummy += "wrench";
			break;
		case 3:
			dummy += "pda_engineer_build";
			break;
		case 4:
			dummy += "pda_engineer_destroy";
			break;
		case 5:
			dummy += "grapplinghook";
			break;
		default:
			dummy += "base";	//can also use "passtime_gun"
			break;
		};
		GTFW.DevPrint(0,"AddWeapon","Created : " + dummy);
		
		weapon = SpawnEntityFromTable(dummy,{})
		
		SetPropInt(weapon, "m_bValidatedAttachedEntity", 1)
		SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemindex)
		SetPropInt(weapon, "m_AttributeManager.m_Item.m_bInitialized", 1)
		
		SetPropEntityArray(ply, "m_hMyWeapons", weapon, slot + GLOBAL_WEAPON_COUNT)
		
		local gtfw_cleanup_wep = weapon;
		
		weapon=SpawnEntityFromTable(classname, {})
		weapon.KeyValueFromString("classname", dummy)

		SetPropInt(weapon, "m_bValidatedAttachedEntity", 1)
		SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemindex)
		SetPropInt(weapon, "m_AttributeManager.m_Item.m_bInitialized", 1)
		
		ply.Weapon_Equip(weapon)
		SetPropEntityArray(ply, "m_hMyWeapons", weapon, slot)
		weapon.ValidateScriptScope()
		weapon.GetScriptScope().GTFW_parent_weapon <- gtfw_cleanup_wep	//for KillWepAll() under _util.nut
	}
	else {
	//	SetPropEntity(weapon, "m_hOwner", ply);
		weapon.DispatchSpawn()	//Dispatches weapon into the world
		ply.Weapon_Equip(weapon)
	}
	
//	DoEntFire("!self", "SetParent", "!activator", 0, ply, weapon);

//	itemindex = GetItemID(itemindex,ForceNotCustom)	//clears custom bits

// Seems that extra wearables now spawn correctly when spawned in via weapon
	local extraWearable = GetPropEntity(weapon, "m_hExtraWearable");
	if ( extraWearable || gtfw_cleanup_wep ) {
		ply.ValidateScriptScope();
		local ply_scope = ply.GetScriptScope();
		if ( !( "GTFW_wearables" in ply_scope ) ) ply_scope.GTFW_wearables <- [];
		ply_scope.GTFW_wearables.append(extraWearable);
		ply_scope.GTFW_wearables.append(gtfw_cleanup_wep);
	}
	
//Engy Toolbox
	if ( classname == "tf_weapon_builder" )	//IDX 28
	{
		SetPropInt(weapon, "BuilderLocalData.m_iObjectType", 2)
		SetPropInt(weapon, "m_iObjectMode", 0)
		SetPropInt(weapon, "m_iSubType", 0)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.000", 1)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.001", 1)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.002", 1)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.003", 0)
	
	}
//Sappers
	if ( classname == "tf_weapon_sapper" ) // IDX 735 and 736 are classname tf_weapon_builder in items_game.txt; opted to change them to tf_weapon_sapper in tables.nut for QoL
	{
		GTFW.DevPrint(0,"AddWeapon", format("Classname %s is updating ",classname));
		
		SetPropInt(weapon, "BuilderLocalData.m_iObjectType", 3)
		SetPropInt(weapon, "m_iObjectMode", 0)
		SetPropInt(weapon, "m_iSubType", 3)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.000", 0)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.001", 0)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.002", 0)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.003", 1)
	}
	

// Seems without this, we can't hit Engineer objects!
	local solidFlags = GetPropInt(weapon, "m_Collision.m_usSolidFlags");
	SetPropInt(weapon, "m_Collision.m_usSolidFlags", solidFlags | Constants.FSolid.FSOLID_NOT_SOLID);
	
	solidFlags = GetPropInt(weapon, "m_Collision.m_usSolidFlags");
	SetPropInt(weapon, "m_Collision.m_usSolidFlags", solidFlags & ~(Constants.FSolid.FSOLID_TRIGGER));
	
	weapon.ReapplyProvision()	// then applies any body attributes back onto the player.
	

	GTFW.DevPrint(0,"AddWeapon", "Initialized \x22"+weapon+"\x22.")
	return weapon
}

//-----------------------------------------------------------------------------
// Purpose: Fixes ammo.
// There is an invisible tf_wearable that tracks ammo for custom weapons, and DisableWeapon/EnableWeapon
// Same goes for metal ammotype weapons like Widowmaker, Wrenches, PDAs etc
//-----------------------------------------------------------------------------
::GTFW.UpdateAmmoFix <- function(player,weapon,WepTFClass,AmmoType,ClipSize,MaxClipSize,AmmoReserve,MaxAmmoReserve)
{
// throw an error if the weapon doesn't use ammo
	if ( !HasProp(weapon, "LocalWeaponData.m_iPrimaryAmmoType") ) {
		GTFW.DevPrint(1,"UpdateAmmoFix", weapon+" is not compatible with ammo fix!");
		return weapon;
	}
	
	GTFW.DevPrint(0,"UpdateAmmoFix", "Starting "+weapon+" ammo fix!");

// Define variables
	local AmmoFix;
	local aThreshold = 0;
	local aPerClass = 0;
	local playerClass = player.GetPlayerClass();
	local WepC = weapon.GetClassname();
	
	if ( AmmoType > TF_AMMO.NONE && MaxAmmoReserve >= 0 ) {
		
	//grabs handle for AmmoFix (a tf_wearable)
		AmmoFix = GTFW.GetAmmoFix(player);
		
	// Purpose: Updates primary ammo reserve
		if ( AmmoType == TF_AMMO.PRIMARY ) {
			aPerClass = MaxAmmoReserve == -1 ? TF_AMMO_PER_CLASS_PRIMARY[WepTFClass] : MaxAmmoReserve;
			aThreshold = TF_AMMO_PER_CLASS_PRIMARY[playerClass];
			AmmoFix.AddAttribute("hidden primary max ammo bonus", aPerClass.tofloat() / aThreshold.tofloat(), -1);
		
		//Purpose: Finds additional attributes on main weapon, to (hopefully) fix most ammo issues when reloading the weapon in MvM.
			AmmoAttributes <- [
				"hidden primary max ammo bonus",
				"maxammo primary increased",
				"maxammo primary reduced"
			]
			foreach (attrb in AmmoAttributes) {
				AmmoReserve = AmmoReserve * weapon.GetAttribute(attrb,1)
			}
		}
	// Purpose: Updates secondary ammo reserve
		else if ( AmmoType == TF_AMMO.SECONDARY ) {
			aPerClass = MaxAmmoReserve == -1 ? TF_AMMO_PER_CLASS_SECONDARY[WepTFClass] : MaxAmmoReserve;
			aThreshold = TF_AMMO_PER_CLASS_SECONDARY[playerClass];
			weapon.AddAttribute("hidden secondary max ammo penalty", aPerClass.tofloat() / aThreshold.tofloat(), -1);	
			
		//Purpose: Finds additional attributes on main weapon, to (hopefully) fix most ammo issues when reloading the weapon in MvM.
			AmmoAttributes <- [
				"hidden secondary max ammo bonus",
				"maxammo secondary increased",
				"maxammo secondary reduced"
			]
			foreach (attrb in AmmoAttributes) {
				AmmoReserve = AmmoReserve * weapon.GetAttribute(attrb,1)
			}
		}
	// Purpose: Updates GRENADES1 ammo reserve
		else if ( AmmoType == TF_AMMO.GRENADES1 ) {
			aThreshold = 1;
			AmmoFix.AddAttribute("maxammo grenades1 increased", (MaxAmmoReserve.tofloat() / aThreshold.tofloat()), -1);
			
		//Purpose: Finds additional attributes on main weapon, to (hopefully) fix most ammo issues when reloading the weapon in MvM.
			AmmoReserve = AmmoReserve * weapon.GetAttribute("maxammo grenades1 increased",1)
		}
	// Purpose: Updates metal capacity
		else if ( AmmoType == TF_AMMO.METAL ) {
			aThreshold = 200;
			AmmoFix.AddAttribute("maxammo metal increased", (MaxAmmoReserve.tofloat() / aThreshold.tofloat()), -1);
			
		//Purpose: Finds additional attributes on main weapon, to (hopefully) fix most ammo issues when reloading the weapon in MvM.
			AmmoAttributes <- [
				"maxammo metal increased",
				"maxammo metal reduced"
			]
			foreach (attrb in AmmoAttributes) {
				AmmoReserve = AmmoReserve * weapon.GetAttribute(attrb,1)
			}
		}
		
	// Purpose: Applies all attributes to player
		AmmoFix.ReapplyProvision();
	}
	


// Purpose: Sets ammo reserve
	if ( AmmoType == TF_AMMO.METAL && EntityOutputs.HasAction(AmmoFix, "OnUser3") == false && playerClass != TF_CLASS_ENGINEER ) {
		EntityOutputs.AddOutput(AmmoFix, "OnUser3", "", "", "", 0.0, -1);	//used to flag for Metal ammo so we don't update twice
		SetPropIntArray(player, "m_iAmmo", AmmoReserve, TF_AMMO.METAL);
	}
	else {
		SetPropIntArray(player, "m_iAmmo", AmmoReserve, AmmoType);
	}
	
	
// Purpose: Sets ammo clip
	if ( MaxClipSize >= 0 
	  && ( MaxClipSize.tofloat() / weapon.GetMaxClip1().tofloat()) != 1 ) {
		aThreshold = 1;
		if ( AmmoType == TF_AMMO.PRIMARY
		  || AmmoType == TF_AMMO.SECONDARY ) {
		//Purpose: Finds additional attributes on main weapon, to (hopefully) fix most ammo issues when reloading the weapon in MvM.
			AmmoAttributes <- [
				"clip size penalty",
				"clip size bonus",
				"clip size penalty HIDDEN",
				"clip size bonus upgrade"
			]
			foreach (attrb in AmmoAttributes) {
				MaxClipSize = MaxClipSize * weapon.GetAttribute(attrb,1)
			}
			local atomic = weapon.GetAttribute("clip size upgrade atomic",0)
			if ( atomic ) MaxClipSize += atomic;
		}
		weapon.SetClip1(MaxClipSize);
	}
	
	
	if ( WepC.find("lunchbox")
	  || WepC.find("jar") 
	  || WepC == "tf_weapon_bat_wood"
	  || WepC == "tf_weapon_bat_giftwrap"
	  || WepC == "tf_weapon_cleaver" 
	  || WepC == "tf_weapon_rocketpack" ) {
		SetPropFloat(weapon, "LocalActiveTFWeaponData.m_flEffectBarRegenTime", Time() + 0.03);
	}
	
	
// Only update clip if stats were NOT Wiped
	local are_Weapon_Stats_Wiped = weapon.GetScriptScope().gw_props & WipeAttributes ? true : false;
	if ( are_Weapon_Stats_Wiped ) return weapon;

// It seems like having both helps account for weapons that don't use the atomic upgrade, but atomic seems to supersede the %clipsize penalty for what ever reason
	if ( MaxClipSize >= 0 ) {
		if ( ( MaxClipSize.tofloat() / weapon.GetMaxClip1().tofloat()) != 1 ) {
			weapon.AddAttribute("clip size upgrade atomic", 0 - (weapon.GetMaxClip1() - MaxClipSize), -1);
			weapon.AddAttribute("clip size penalty HIDDEN", MaxClipSize.tofloat() / weapon.GetMaxClip1().tofloat(), -1);
		}
		weapon.SetClip1(MaxClipSize);
	}
	if ( MaxClipSize != ClipSize ) {
		weapon.SetClip1(ClipSize);
	}
	
	GTFW.DevPrint(0,"UpdateAmmoFix", format("%s %s %s","Finished ammo fix!",AmmoType.tostring(),AmmoReserve.tostring()));
	
	return weapon
}

//-----------------------------------------------------------------------------
// Purpose: Creates an invisible tf_wearable with attributes that adjust ammo reserve
//			Also used for AddThinkToView() viewmodel think script when switching weapons/taunting
//-----------------------------------------------------------------------------
::GTFW.GetAmmoFix <- function(player=null) {
	if ( player && player.GetClassname() != "player" ) {
		GTFW.DevPrint(1,"GTFW.GetAmmoFix", "Function failed. Param 1 not a player. Returning null.");
		return null;
	}
	
	local AmmoFix;
	player.ValidateScriptScope();
	
	local plyScope = player.GetScriptScope();
	local ammofix;

	if ( "AMMOFIX" in plyScope ) { 
		ammofix = plyScope.AMMOFIX;
		
		if ( ammofix && ammofix.IsValid() ) return ammofix;
		else delete plyScope.AMMOFIX;
	}
	
	ammofix = player.CreateCustomWearable();
	plyScope.AMMOFIX <- ammofix;

	GTFW.DevPrint(0,"CreateAmmoFix", "Created " + ammofix + " for " + GTFW.GetNetName(player));
	return ammofix;
}


//-----------------------------------------------------------------------------
// Purpose: Used to update viewmodel arms animations in GiveWeapon()
//-----------------------------------------------------------------------------
::GTFW.UpdateArms <- function(player, givenwep=null)
{
// throw error if VM fix not enabled
	if ( !CVAR_USE_VIEWMODEL_FIX ) {
		GTFW.DevPrint(0,"UpdateArms", "CVAR_USE_VIEWMODEL_FIX = false. UpdateArms cancelled.")
		return null
	}
// throw error if weapon not compatible with class arms
// (tf_wearable and other passive items don't have "m_iPrimaryAmmoType" netprop)
	if ( !HasProp(givenwep, "LocalWeaponData.m_iPrimaryAmmoType") ) {
		GTFW.DevPrint(1,"UpdateArms", givenwep+" is not a compatible weapon!")
		return givenwep
	}
	
// Purpose: This is the special VM fix.
// it creates class arms that bonemerge with VM

// Adds a class arms model if nothing has parented to the main_viewmodel
	local main_viewmodel = GetPropEntity(player, "m_hViewModel");
	main_viewmodel.ValidateScriptScope()
	local VM_Scope = main_viewmodel.GetScriptScope();
// We check Gunslinger as Engineer 
	local hasGunslingerAsEngineer = player.HasGunslinger() && player.GetPlayerClass() == TF_CLASS_ENGINEER;
	local arms_in_scope = "classArms" in VM_Scope;
	local arms = arms_in_scope ? VM_Scope.classArms : null;
	
	local playerTFclass = player.GetPlayerClass();
	
	if ( arms_in_scope
	  && arms && arms.IsValid() ) {
		arms.Kill();
	}
	if ( arms_in_scope && ( !arms || !arms.IsValid() ) )
		delete VM_Scope.classArms;

	if ( !( "classArms" in VM_Scope ) ) {
		local arms;
		if ( hasGunslingerAsEngineer )
			arms = GTFW_MODEL_ARMS[10];
		else 
			arms = GTFW_MODEL_ARMS[playerTFclass];
		local classarms_model = player.CreateCustomWearableViewModel(arms, main_viewmodel);

		GTFW.DevPrint(0,"UpdateArms", "Created arms " + classarms_model + " : " + arms);
		
	// Purpose: Puts bonemerged class arms model in main_viewmodel scope for easy tracking
		VM_Scope.classArms <- classarms_model;
	
	// For player.SetCustomClassArms()
		if ( !("model_path" in VM_Scope) ) VM_Scope["model_path"] <- {};
		VM_Scope["model_path"][playerTFclass] <- arms;
	}
	
// Purpose: 
	givenwep.ValidateScriptScope()
	local wepScope = givenwep.GetScriptScope();
	local wepID = wepScope.itemID;
	local model;
		
	if ( type( wepScope.classArms ) == "string" && wepScope.classArms.len() )
		model = wepScope.classArms;
	else if ( wepScope.tf_class == 0 || player.GetPlayerClass() == wepScope.tf_class )
		model = GTFW_MODEL_ARMS[player.GetPlayerClass()];
	
	if ( !model || type(model) != "string" ) model = GTFW_MODEL_ARMS[wepScope.tf_class];

	PrecacheModel( model );
	local modelIndex = GetModelIndex( model );
	givenwep.SetModelSimple(model);
	givenwep.SetCustomViewModelModelIndex(modelIndex);
	SetPropInt(givenwep, "m_iViewModelIndex", modelIndex);
	
	GTFW.DevPrint(0,"UpdateArms", "Updated weapon " + givenwep + " with model " + model);
}


//-----------------------------------------------------------------------------
//	Purpose: Updates all weapon models for custom/unintended for tfclass weapons
//	This function works with AddThinkToViewModel() to make weapons visible/invisible as needed.
//   Things like weapon switching, taunting, etc, will make the replaced weapon invisible and make the vscript custom weapon visible.
//-----------------------------------------------------------------------------
 
::GTFW.UpdateCustomWeapon <- function(player, weapon, worldModel, viewModel)
{
// Throw error if param player isn't valid
	if ( type( player ) != "instance" || ( player && player.GetClassname() != "player" ) ) {
		GTFW.DevPrint(1,"UpdateCustomWeapon", "Function failed. Parameter 1 is not a player. Returning null.");
		return null;
	}
	if ( type( weapon ) != "instance" ) {
		GTFW.DevPrint(1,"UpdateCustomWeapon", "Function failed. Parameter 2 is not a weapon. Returning null.");
		return null;
	}
	if ( type( worldModel ) != "integer" && type( worldModel ) != "string" ) {
		GTFW.DevPrint(1,"UpdateCustomWeapon", "Function failed. Parameter 3 is not an int or string. Returning null.");
		return null;
	}
	if ( type( viewModel ) != "integer" && type( viewModel ) != "string" ) {
		GTFW.DevPrint(1,"UpdateCustomWeapon", "Function failed. Parameter 4 is not an int or string. Returning null.");
		return null;
	}
	
// Define variables
	local WepC = weapon.GetClassname();
	local Is_DemoShield = WepC == "tf_wearable_demoshield";
	local hasGunslingerAsEngineer =  WepC == "tf_weapon_robot_arm" && player.GetPlayerClass() == TF_CLASS_ENGINEER && !IS_CUSTOM(weapon);
	local hasFists = WepC == "tf_weapon_fists" && !IS_CUSTOM(weapon);
	
// Purpose: Only update if the weapon is custom, or a demoshield
// filters out non-custom Gunslingers and Fists because those don't need extra models
	if ( hasGunslingerAsEngineer 
	  || hasFists ) { return weapon; }
	else if ( IS_CUSTOM(weapon)		// If weapon is custom
	  || Is_DemoShield )			// or a demoshield
	{
		GTFW.DevPrint(0,"UpdateCustomWeapon", "Updating \x22"+weapon+"\x22 with " + viewModel);
		
		if ( viewModel != "models/empty.mdl" ) {
			if ( WepC == "tf_wearable_demoshield" ) {
				local main_viewmodel = GetPropEntity(player, "m_hViewModel");
				player.CreateCustomWearableViewModel( viewModel, main_viewmodel)
			}
			else {
				player.CreateCustomWearableViewModel( viewModel, weapon)
			}
		}
		
		if ( worldModel != "models/empty.mdl" )
			player.CreateCustomWearableWeaponWorldModel("tf_wearable",worldModel, null, weapon);
	//for custom weapons, this makes the weapon it's replacing invisible.
//		DoEntFire("!self", "Clearparent", "", 0, null, weapon);
//		DoEntFire("!self", "DisableShadow", "", 0, null, weapon);
	//	weapon.AcceptInput("DisableShadow", "", weapon, weapon);
	}
	
	return weapon
}


//-----------------------------------------------------------------------------
// Purpose: Fixes class animations  for world model
//-----------------------------------------------------------------------------

::GTFW.UpdateClassAnimations <- function(player, GivenWeapon)
{
	if ( !GivenWeapon || !GivenWeapon.IsValid() ) return;
	
	local playerDisplayModel;
	local playerscope = player.GetScriptScope();
	local wepScope = GivenWeapon.GetScriptScope();

	if ( wepScope.gw_props & (FixClassAnims|FixClassAnimsRef|FixClassAnimsITEM) ) {
		
		playerDisplayModel = "GTFW_classAnimsModel" in playerscope ? playerscope.GTFW_classAnimsModel : null;
		if ( !playerDisplayModel || !playerDisplayModel.IsValid() ) playerDisplayModel = null;
		
		local playerOriginalClassModel = "GTFW_playerOriginalClassModel" in playerscope ? playerscope.GTFW_playerOriginalClassModel : null;
		
		local model_path;
		if ( playerOriginalClassModel ) model_path = playerOriginalClassModel;
		else if ( playerDisplayModel ) model_path = playerDisplayModel.GetModelName();
		else model_path = player.GetModelName();
		
		if ( !playerDisplayModel )  
			playerDisplayModel = GTFW.CreateTFClassModel(player, model_path);
			
		playerDisplayModel.SetModelSimple(model_path);
		
		SetPropInt(player, "m_nRenderMode", 2);
	}
	return playerDisplayModel
}

//-----------------------------------------------------------------------------
// Purpose: Updates viewmodel think script.
// For fixing tf_wearable being invisible when taunting, and for custom hands on custom weapons + unintended class weapons
//-----------------------------------------------------------------------------

::GTFW.InThirdPerson <- function(player)
{
	if ( GetPropInt(player,"m_nForceTauntCam")
	// || player.IsTaunting()
	 //|| IsPlayerABot(player) 
	 || ( IsInMedievalMode() && Convars.GetInt( "tf_medieval_thirdperson" ) ) )
		 return true;
	return false;
}

::GTFW.SetWeaponModel_OFF <- function(wep) {
	
	if ( !wep || !wep.IsValid() ) return;
	
	wep.ValidateScriptScope();
	local wepScope = wep.GetScriptScope();
	local hasWorldModelProp = "worldModelProp" in wepScope ? wepScope.worldModelProp : false;
	local hasViewModelProp = "viewModelProp" in wepScope ? wepScope.viewModelProp : false;
	
	wep.AcceptInput("DisableShadow", "", wep, wep);
	if ( hasWorldModelProp && hasWorldModelProp.IsValid() ) {
		wep.KeyValueFromInt("effects",161);
		hasWorldModelProp.DisableDraw();
	}
	//DoEntFire("!self", "SetParent", "!activator", 0.0, player, wep);
}

::GTFW.AddThinkToViewModel <- function(player, playerDisplayModel)
{
	if ( !CVAR_USE_VIEWMODEL_FIX ) {
		GTFW.DevPrint(0,"AddThinkToViewModel", "CVAR_USE_VIEWMODEL_FIX set. Ignoring function.");
		return;
	}
	
	GTFW.DevPrint(0,"AddThinkToViewModel", "Function started.");
	
// Think Script stuff

	local wepScope;
	player.ValidateScriptScope();
	local playerscope = player.GetScriptScope();

	local wepC, last_active_building, player_tfclass;
	local wep, current_weapon;
	local hasWorldModelProp, hasViewModelProp;
	
	const THINK_VMFIX_DELAY = -1;
	
	local Taunting = true;
	local ThirdPerson = true;
	local JustEnteredThirdPerson = true;
	local JustTaunted = true;
	local JustSwitchedWeapons = true;
	
// Think script itself
// It checks for whenever you go in or out of third person, based on:
// if you just taunted, or if you just used SetForceTauntCam(),
// or just switched weapons.
// Kind of a mess to read, sorry about that.

	local main_viewmodel = GetPropEntity(player, "m_hViewModel");
	main_viewmodel.ValidateScriptScope();
	local VM_Scope = main_viewmodel.GetScriptScope();
	
	local entscriptname = "THINK_VM_FIX";
	VM_Scope[entscriptname] <- function() {
	
		current_weapon = player.GetActiveWeapon();

		if ( !current_weapon || !current_weapon.IsValid() ) return THINK_VMFIX_DELAY; //makes sure if in ref pose, don't error
		
// test for if player is in third person, is taunting, or they switched weapons
		ThirdPerson = GTFW.InThirdPerson(player);
		Taunting = player.IsTaunting();
		JustSwitchedWeapons = current_weapon != wep;
		
// after finishing taunting, make it run firstperson code
		if ( JustTaunted && !Taunting ) {
			//printl(" taunted: reset");
			JustTaunted = false;
			JustSwitchedWeapons = true;
		}
	
// after exiting thirdperson, reset variable
		if ( !JustEnteredThirdPerson && !ThirdPerson ) {
			//printl(" thirdperson: reset");
			JustEnteredThirdPerson = true;
		}
	
// if entered into thirdperson, or player is taunting, run this code ONCE
// This part runs fixes world model stuff--Turning off weapon visibilitity for taunting, turning off bonemerged class anims model for taunting, and fixing tf_wearable going invisible while in thirdperson.
		if ( ( JustEnteredThirdPerson && ThirdPerson ) 
		  || ( !JustTaunted && Taunting ) 
		) {
	//run only taunt part of the code here
			if ( !JustTaunted && Taunting ) {
				//printl(" taunted");
				JustTaunted = true;
				
				if ( playerDisplayModel && playerDisplayModel.IsValid() 
				  && wepScope.gw_props & (FixClassAnims|FixClassAnimsRef|FixClassAnimsITEM) ) {
					GTFW.SetUseClassAnimations_OFF(player,playerscope, playerDisplayModel, wepScope, playerscope.GTFW_playerOriginalClassModel);
				}
		// Fixes taunt props
			// turn off custom weapons while taunting
				if ( hasWorldModelProp && hasWorldModelProp.IsValid() ) {
					wep.KeyValueFromInt("effects",161);
					hasWorldModelProp.DisableDraw();
		//			SetPropInt(wep, "m_nModelIndexOverrides", 0)		//fixes taunt props, but settings this for weapons makes first person animations lag
				}
			}
	// after entering from firstperson -> thirdperson, make sure we don't execute this code again
			if ( JustEnteredThirdPerson && ThirdPerson ) {
				//printl(" thirdperson");
				JustEnteredThirdPerson = false;
			}
	
	// thirdperson code
			//printl(" run code");
			
			wepScope = current_weapon.GetScriptScope();
		
	// tf_wearables go invisible if they aren't in sight range of the player (in this case, entering thirdperson or taunting)
		// We fix it by going through each child on player, check if they're tf_wearable, then makes them invisible then visible with a delay
			local wearables_list = [];
			for ( local item = player.FirstMoveChild(); item != null ; item = item.NextMovePeer() )
			{
				 if ( startswith(item.GetClassname(),"tf_wearable") )
					wearables_list.append(item)
			}
			
		// Turning "m_bInitialized" off and on fixes the tf_wearables being invisible in thirdperson
		// and I think resetting model scale does help somehow too? I honestly forgot!!
			foreach (item in wearables_list)
			{
				SetPropInt(item, "m_AttributeManager.m_Item.m_bInitialized", 0);
				EntFireByHandle(item, "RunScriptCode", "SetPropInt(self, \x22m_AttributeManager.m_Item.m_bInitialized\x22, 1)", 0.09, item, item);
				item.SetModelScale(item.GetModelScale(),0);
			}
		}
		
// first person fixup code
// execute this code by: if we switched weapons (aka "current weapon was not last weapon"), or if the Engineer's building was different from last
		if ( JustSwitchedWeapons
		  || ( HasProp(wep, "m_hObjectBeingBuilt") && GetPropEntity(wep, "m_hObjectBeingBuilt") != last_active_building )
		) {
			JustTaunted = false;
			JustEnteredThirdPerson = false;
			
			//GTFW_printl("WEP SWITCHED PROCESSED");
	
	// First part removes visibility of the last weapon being held
			if ( !wep || !wep.IsValid() ) wep = current_weapon;
			GTFW.SetWeaponModel_OFF(wep);	//function above AddThinkToView()
			
	// Second part enables the current weapon being held
			wep = current_weapon;
			wep.ValidateScriptScope();
			wepScope = wep.GetScriptScope();
			hasWorldModelProp = "worldModelProp" in wepScope ? wepScope.worldModelProp : false;
			hasViewModelProp = "viewModelProp" in wepScope ? wepScope.viewModelProp : false;
			wepC = wep.GetClassname();
			player_tfclass = player.GetPlayerClass();
			
		// Fixes Engineer's Toolbox class arms by setting last building instead of last weapon
			if ( HasProp(wep, "m_hObjectBeingBuilt") )
				last_active_building = GetPropEntity(wep, "m_hObjectBeingBuilt");

			if ( hasWorldModelProp && hasWorldModelProp.IsValid() ) {
		//		SetPropInt(wep, "m_fEffects", 32);
		//		SetPropInt(wep, "m_nModelIndexOverrides", GetModelIndex("models/empty.mdl"));
		//		DoEntFire("!self", "Clearparent", "", 0.1, player, wep);
				wep.KeyValueFromInt("effects",161);
				hasWorldModelProp.EnableDraw();
				DoEntFire("!self", "DisableShadow", "", 0.1, null, wep);
			}
		
	// Fixes class arms and custom weapon models
	// Class arms by default are attached to the weapon, so disabling visibility on them disables visibility on the weapon too.
	// Earlier we created a bonemerged class arms prop (tf_wearable_vm) to the arms.
	// We don't need to enable the weapon like the third person models, because we have two netprops that do it for us:
		// "m_hWeaponAssociatedWith" on tf_wearable_vm,
		// and "m_hExtraWearableViewModel" on tf_weapon_*'s
			if ( hasViewModelProp && hasViewModelProp.IsValid() ) {
				//Checks for custom weapon and disables only non-gunslinger and non-fists
				local hasFistsAsHeavy = wepC == "tf_weapon_fists" && player_tfclass == TF_CLASS_HEAVYWEAPONS;
				local hasGunslingerAsEngineer = wepC == "tf_weapon_robot_arm" && player_tfclass == TF_CLASS_ENGINEER;
				
				if ( !hasFistsAsHeavy || !hasGunslingerAsEngineer )
				{
					main_viewmodel.DisableDraw();		//makes firstperson default weapon/class arms invisible. We already bonemerged our own, so this makes sure they don't overlap
				}
			}
			if ( "classArms" in VM_Scope ) {
				local classArms = VM_Scope.classArms;
				
				if ( classArms && classArms.IsValid() ) {
				// if using weapon Short Circuit, disable the class arms' visibility
					local using_ShortCircuit = wepC == "tf_weapon_mechanical_arm";
					
					if ( wep && wep.IsValid() ) {
						 if ( using_ShortCircuit )
							classArms.DisableDraw();
						else
							classArms.EnableDraw();
					}
				}
			}
		
	// updates player class model animations + bonemerged class model, as long as gtfw settings are enabled for the weapon and model exists
			if ( playerDisplayModel && playerDisplayModel.IsValid() ) {
				local WeaponHasClassAnimsFix = wepScope.gw_props & FixClassAnims;
				local WeaponHasRefPoseFix = wepScope.gw_props & FixClassAnimsRef;
				local WeaponHasITEMFix = wepScope.gw_props & FixClassAnimsITEM;
				local activityName = player.GetSequenceActivityName(player.GetSequence());
				if ( ( WeaponHasClassAnimsFix )
				  || ( WeaponHasRefPoseFix && player.GetSequence() == 0 )
				  || ( WeaponHasITEMFix && wepScope.tf_class != player_tfclass && activityName.find("ITEM", 7) ) ) {
					GTFW.SetUseClassAnimations_ON(player,playerscope, playerDisplayModel, wepScope, wepScope.classAnimsModel);
				}
				else {
					GTFW.SetUseClassAnimations_OFF(player,playerscope, playerDisplayModel, wepScope, playerscope.GTFW_playerOriginalClassModel);
				}
			}
		}
		
		
	// return to beginning of think function
		return THINK_VMFIX_DELAY;
		
	}
	AddThinkToEnt(main_viewmodel, entscriptname);	//adds think script
	
// debug msg
	GTFW.DevPrint(0,"AddThinkToViewModel", "Success! Added think script to "+main_viewmodel);
}	


//unused: framework for getting taunts / thirdperson code to work
//		  It's split into two different segments. 1st commented out gets it all running, the followed 2nd part combines it so we can keep the code consolidated in one area without a separate function.

function gtfw_old_AddThinkToView()	
{	
	VM_Scope[entscriptname] <- function() {
	print("z");
		local playerIsInThirdPerson = GTFW.InThirdPerson(player);
		local playerIsTaunting = player.IsTaunting();
	
// basic test
	/* 
		if ( playerJustTaunted && !playerIsTaunting ) {
			printl(" taunted: reset");
			playerJustTaunted = false;
		}
	
		if ( !playerJustTaunted && playerIsTaunting ) {
			printl(" taunted: run code");
			playerJustTaunted = true;
		}
		
		if ( playerJustEnteredThirdPerson && playerIsInThirdPerson ) {
			printl(" thirdperson: run code");
			playerJustEnteredThirdPerson = false;
		}
	
		if ( !playerJustEnteredThirdPerson && !playerIsInThirdPerson ) {
			printl(" thirdperson: reset");
			playerJustEnteredThirdPerson = true;
		}
	*/
// combined test
		if ( JustTaunted && !Taunting ) {
			printl(" taunted: reset");
			JustTaunted = false;
		}
	
		
		if ( ( JustEnteredThirdPerson && ThirdPerson ) 
		  || ( !JustTaunted && Taunting ) 
		) {
			if ( !JustTaunted && Taunting ) {
				printl(" taunted");
				JustTaunted = true;
			}
			if ( JustEnteredThirdPerson && ThirdPerson ) {
				printl(" thirdperson");
				JustEnteredThirdPerson = false;
			}
			printl(" run code");
		}
	
		if ( !JustEnteredThirdPerson && !ThirdPerson ) {
			printl(" thirdperson: reset");
			JustEnteredThirdPerson = true;
		}

	}
	
	AddThinkToEnt(main_viewmodel, entscriptname);	//adds think script
}

