//-----------------------------------------------------------------------------
// Purpose: Pre-Fixes up all items when found by handle.GetWeaponTable().
// It shoves everything into a table.
//-----------------------------------------------------------------------------
::GTFW.PreWepFix <- function(table, ID)
{
	//all of our properties
	local new_table = {
		weapon			=	null,
		tf_class		=	0,
		slot			=	0,
		classname		=	"tf_weapon_base",
		itemID			=	65535,
		name			=	"",
		ammoType		=	0,
		clipSize		=	-1,
		clipSizeMax		=	-1,
		reserve			=	-1,
		reserveMax		=	-1,
		worldModel		=	"",
		viewModel		=	"",
		wearable		=	"",
		wearable_vm		=	"",
		gw_props		=	0,
		func			=	null,
		classArms		=	"",
		PostWepFix		=	function(a,b){},
		classAnimsModel	=	"",
		
		announce_quality	=	20,
		announce_prefix		=	null,
		announce_has_string	=	null,
	}
		
	if ( "tf_class" in table )
		new_table.tf_class = table.tf_class;
	
	if ( "func" in table )
		new_table.func = table.func;

	if ( "worldModel" in table && "viewModel" in table ) {
		new_table.worldModel = table.worldModel;
		new_table.viewModel = table.viewModel;
	}
	else if ( "worldModel" in table && !("viewModel" in table) ) {
		new_table.worldModel = table.worldModel;
		new_table.viewModel = table.worldModel;
	}
	else if ( "viewModel" in table && !("worldModel" in table) ) {
		new_table.worldModel = table.viewModel;
		new_table.viewModel = table.viewModel;
	}

	if ( "classArms" in table )
		new_table.classArms = table.classArms;
	
	if ( "classAnimsModel" in table )
		new_table.classAnimsModel = table.classAnimsModel;
	else if ( new_table.tf_class == 0 ) new_table.classAnimsModel = "";
	else new_table.classAnimsModel = GTFW_MODEL_TFCLASSES[new_table.tf_class];

	new_table.name		= table.name;
	new_table.slot		= table.slot;
	new_table.classname	= table.classname;
	new_table.itemID	= ID;
	new_table.ammoType	= table.ammoType;
	
	
	new_table.clipSize	= table.clipSize;
	if ( "clipSizeMax" in table )
		new_table.clipSizeMax	= table.clipSizeMax;
	else
		new_table.clipSizeMax	= table.clipSize;
	
	new_table.reserve	= table.reserve;
	if ( "reserveMax" in table )
		new_table.reserveMax	= table.reserveMax;
	else
		new_table.reserveMax	= table.reserve;
	
	
	new_table.wearable	= table.wearable;
	
	if ( "wearable_vm" in table )
		new_table.wearable_vm	= table.wearable_vm;
	
	// gw_props stands for GiveWeapon PROPertieS
	// It's a bitmask that gives extra customization for weapons
	// i.e. if the weapon replaces the old one in slot, if it auto switches when obtained, if the weapon is announced in chat, etc.
	// "Defaults" is defined in `_master.nut`
	if ( "gw_props" in table ) {
		if ( type(table.gw_props) != "integer")
			table.gw_props = 0;
		new_table.gw_props = table.gw_props;
	}
		
	
	//AnnounceToChat bitmask
	 // item quality; color
	if ( "announce_quality" in table )
		new_table.announce_quality = table.announce_quality;
	 // item prefix; i.e. "Unusual", "Haunted", "Collector's", etc.
	if ( "announce_prefix" in table )
		new_table.announce_prefix = table.announce_prefix;
	 // item "has found:" string
	if ( "announce_has_string" in table )
		new_table.announce_has_string = table.announce_has_string;


	// Post Wep Fix, for changing any other attributes
	if ( "PostWepFix" in table )
		new_table.PostWepFix = table.PostWepFix;
	
	return new_table
}


//-----------------------------------------------------------------------------
// Purpose: Post-Fixes up all items found by handle.GetWeaponTable.
// Things like what the classname is, what type of shotgun it is, etc.
// Supports inserting custom function using RegisterCustomWeapon()
//-----------------------------------------------------------------------------
::GTFW.PostWepFix <- function(wepScope, ply)
{
	if ( wepScope == null) {
		return;
	}
	
// Define variable
	local WepC = wepScope.classname;
	local ItemID = GetItemID(wepScope.itemID,ForceNotCustom);
	local tfPlayerClass = ply.GetPlayerClass();
	
	if ( WepC == "demoshield" )
		wepScope.classname = "tf_wearable_demoshield";
	else if ( WepC == "razorback" )
		wepScope.classname = "tf_wearable_razorback";
	else if ( WepC == "tf_wearable" )
		wepScope.classname = WepC;
	else if ( WepC == "saxxy" )
		wepScope.classname = GTFW_Saxxy[tfPlayerClass];
	else if ( WepC == "pistol"
		  ||  WepC == "handgun_scout_secondary" )
		{
	// All-Class Winger
		if ( ItemID == 449 ) {
			wepScope.name		= "Winger";
		}
	// All-Class Pocket Pistol
		else if ( ItemID == 773 ) {
			wepScope.name		= "Pretty Boy's Pocket Pistol";
		}
		
		if ( tfPlayerClass == TF_CLASS_SCOUT ) {
			wepScope.classname	= "pistol_scout";
			wepScope.reserve	= 36;
			wepScope.tf_class 	= TF_CLASS_SCOUT;
		}
		else if ( tfPlayerClass == TF_CLASS_ENGINEER ) {
			wepScope.classname	= "pistol";
			wepScope.reserve	= 200;
			wepScope.tf_class 	= TF_CLASS_ENGINEER;
		}
		else {
			wepScope.classname	= "pistol";
			wepScope.reserve	= 36;
			wepScope.tf_class 	= TF_CLASS_ENGINEER;
		}
	}
	else if ( WepC == "pistol_scout" ) {
		if ( tfPlayerClass == TF_CLASS_ENGINEER ) {
			wepScope.classname	= "pistol_scout";
		//	wepScope.reserve	= 200;
			wepScope.tf_class 	= TF_CLASS_ENGINEER;
		}
	}
	else if ( WepC == "shotgun" ) {
	//Fixups for All-Class Shotguns
		if ( tfPlayerClass == TF_CLASS_ENGINEER ) {
			wepScope.classname	= "shotgun_primary";
			wepScope.slot = 0;
			wepScope.tf_class 	=	TF_CLASS_ENGINEER;
			wepScope.ammoType	=	TF_AMMO.PRIMARY;
		}
		else if ( tfPlayerClass == TF_CLASS_HEAVYWEAPONS ) {
			wepScope.classname	= "shotgun_hwg";
			wepScope.tf_class 	=	TF_CLASS_HEAVYWEAPONS;
		}
		else if ( tfPlayerClass == TF_CLASS_SOLDIER ) {
			wepScope.classname	= "shotgun_soldier";
			wepScope.tf_class 	= TF_CLASS_SOLDIER;
		}
		else if ( tfPlayerClass == TF_CLASS_PYRO ) {
			wepScope.classname	= "shotgun_pyro";
			wepScope.tf_class 	=	TF_CLASS_PYRO;
		}
		else if ( tfPlayerClass == TF_CLASS_DEMOMAN ) {
			wepScope.classname	= "shotgun_soldier";
			wepScope.tf_class 	=	TF_CLASS_DEMOMAN;
		}
		else if ( tfPlayerClass == TF_CLASS_MEDIC ) {
			wepScope.classname	= "shotgun_primary";
			wepScope.slot = 0;
			wepScope.tf_class 	=	TF_CLASS_ENGINEER;
			wepScope.ammoType	=	TF_AMMO.PRIMARY;
		}
		else if ( tfPlayerClass == TF_CLASS_SPY ) {
			wepScope.classname	= "shotgun_primary";
			wepScope.slot = 0;
			wepScope.tf_class 	=	TF_CLASS_ENGINEER;
			wepScope.ammoType	=	TF_AMMO.PRIMARY;
		}
		else if ( tfPlayerClass == TF_CLASS_SCOUT ) {
			wepScope.classname	= "shotgun_primary";
			wepScope.slot = 0;
			wepScope.tf_class 	=	TF_CLASS_ENGINEER;
			wepScope.ammoType	=	TF_AMMO.PRIMARY;
		}
		else {
			wepScope.classname	= "shotgun_pyro";
			wepScope.tf_class 	=	TF_CLASS_PYRO;
		}
	}
	else if ( WepC == "katana" ) {
		if ( tfPlayerClass == TF_CLASS_DEMOMAN ) {
			wepScope.classname = "katana";
			wepScope.tf_class 	=	TF_CLASS_DEMOMAN;
		}
	}
	else if ( WepC == "revolver" ) {
		if ( tfPlayerClass != TF_CLASS_SPY ) {
			wepScope.classname = "revolver";
			wepScope.ammoType	=	TF_AMMO.PRIMARY;
		}
	}
	else if ( WepC == "parachute" || WepC == "parachute_primary" || WepC == "parachute_secondary" ) {
		if ( ItemID == 1101 ) {
			wepScope.name		= "B.A.S.E. Jumper";	//updates announce quality name
		}
	}
// Beggar's Bazooka
	else if ( ItemID == 730 )
		wepScope.clipSize = 0;
// Horseless Horseman's Headtaker
	else if ( ItemID == 266 )
		wepScope.announce_quality = 5;	// Unusual quality (for HHHH)
// Disguise Kit
	else if ( ItemID == 27 )
		wepScope.classArms = "models/weapons/v_models/v_pda_spy.mdl";
// Invisibility Watches
	else if ( WepC == "invis" ) {
		if ( ItemID == 30 )
			wepScope.classArms = "models/weapons/v_models/v_watch_spy.mdl";
		else if ( ItemID == 59 )
			wepScope.classArms = "models/weapons/v_models/v_watch_pocket_spy.mdl";
		else if ( ItemID == 60 )
			wepScope.classArms = "models/weapons/v_models/v_watch_leather_spy.mdl";
		else if ( ItemID == 297 )
			wepScope.classArms = "models/weapons/v_models/v_ttg_watch_spy.mdl";
		else if ( ItemID == 947 )
			wepScope.classArms = "models/workshop_partner/weapons/v_models/v_hm_watch/v_hm_watch.mdl";
	}
// Gunslinger
	else if ( WepC == "robot_arm" )
		wepScope.classArms = GTFW_MODEL_ARMS[10];

//Stock items
	if ( ItemID <= 30 )
		wepScope.announce_quality = 0;	// gray quality (for stock items)
		
// If making a custom item, run this function
	if ( wepScope.PostWepFix ) {
		wepScope.PostWepFix(wepScope, ply);
	}

// Adds prefix "tf_weapon_" if unavailable
	if ( !wepScope.classname.find("wearable") ) 
		wepScope.classname = wepScope.classname.find("weapon") ? wepScope.classname : "tf_weapon_" + wepScope.classname;

	return wepScope
}
