//-----------------------------------------------------------------------------
// Purpose: Shows in console all equips on host of listen server.
//	-> See top of script for more info
//-----------------------------------------------------------------------------

::CheckItems <- function()
{
	local ActiveWeapon = GetListenServerHost().GetActiveWeapon()
	local ActiveWeaponID = GetItemID(ActiveWeapon)

	Say(GetListenServerHost(), " ", false)
	Say(GetListenServerHost(), format("Active Slot%i [%s] (ItemID = %i)", ActiveWeapon.GetSlot(), ActiveWeapon.GetClassname(), ActiveWeaponID), false)
	
	for (local i = 0; i < 47; i++)
	{
		local wep = GetPropEntityArray(GetListenServerHost(), "m_hMyWeapons", i)
		local wep_itemID = GetItemID(wep)
		ClientPrint(GetListenServerHost(), 2, i+" "+wep+" (ItemID = "+wep_itemID+")" )
		
		if ( wep && wep != ActiveWeapon)
		{
			Say(GetListenServerHost(), format("Slot%i [%s] (ItemID = %i)", wep.GetSlot(), wep.GetClassname(), wep_itemID), false)
		}
	}
}


//-----------------------------------------------------------------------------
// Purpose: Send error with time
//-----------------------------------------------------------------------------
::GTFW_printl <- function(Message="") {
	printl(format("(%f) %s",Time(),Message.tostring()));
}



//-----------------------------------------------------------------------------
// Purpose: Prints error messages if CVAR_GTFW_DEBUG_MODE = true
//-----------------------------------------------------------------------------
::GTFW.DevPrint <- function(error=false,func="",message="")
{
	if ( !CVAR_GTFW_DEBUG_MODE ) return;
	
	if ( error )
		error = "\tERROR ";
	else error = ""
		
	//if ( func != "CreateCustomWearable") return;
	printl( format("g_tf_w %s%s: %s",error.tostring(),func.tostring(),message.tostring() ) );
}




//-----------------------------------------------------------------------------
// Purpose: Grabs net name for player
//-----------------------------------------------------------------------------
::GTFW.GetNetName <- function(player) {
	if ( HasProp(player, "m_szNetname") )
		return GetPropString(player, "m_szNetname");
	return player;
}

//-----------------------------------------------------------------------------
// Purpose: Used by events to clean up any unused entities made by this script.
//-----------------------------------------------------------------------------
::CTFPlayer.GTFW_Cleanup <- function()
{
	GTFW.DevPrint(0,"GTFW_Cleanup", "Running cleanup for "+GTFW.GetNetName(this));
	
	local player = this;
	player.ValidateScriptScope()
	local playerScope = player.GetScriptScope();
	
// Updates class models
//	player.SetCustomModelWithClassAnimations(GTFW_MODEL_TFCLASSES[hPlayer.GetPlayerClass()])
	
	
	// Deletes any viewmodels that the script has created
	local main_viewmodel = GetPropEntity(player, "m_hViewModel");
	AddThinkToEnt(main_viewmodel, null);	//clears script if it was being used
	if ( main_viewmodel && main_viewmodel.IsValid() ) {
		if ( player.HasGunslinger() && player.GetPlayerClass() == 9 )
			main_viewmodel.SetModelSimple( GTFW_MODEL_ARMS[10] );
		else 
			main_viewmodel.SetModelSimple( GTFW_MODEL_ARMS[player.GetPlayerClass()] );
	}

// restores player alpha if world model class animations fix was set to true
	if ( "GTFW_rendermode" in playerScope 
	  && playerScope["GTFW_rendermode"] == true ) {
		GTFW.SetUseClassAnimations_OFF(player,playerScope);
	}

//clears all custom wearables in player scope
// ammofix already in GTFW_wearables list, so just delete it from scope here
	if ( "AMMOFIX" in playerScope ) {
		delete playerScope["AMMOFIX"];
	}
// goes through each item that was in the list, and deletes them
	if ( "GTFW_wearables" in playerScope ) {
		local wearableArray = playerScope.GTFW_wearables;
		foreach(k, item in wearableArray) {
			if ( item && item.IsValid() ) {
				GTFW.DevPrint(0,"GTFW_Cleanup", "Deleted "+item);
				
				item.Kill();
			}
		}
		delete playerScope.GTFW_wearables;
	}
	
	if ( "GTFW_classAnimsModel" in playerScope ) {
		local cleanup = playerScope.GTFW_classAnimsModel;
		if ( cleanup && cleanup.IsValid() ) cleanup.Kill();
		delete playerScope.GTFW_classAnimsModel;
	}
	
// "tf_wearable_robot_arm" from Gunslinger keeps duplicating everytime the Gunslinger is made.
//	It doesn't seem to do anything visually, so I'm deleting it permanently.
	for (local robot_arm = player.FirstMoveChild(); robot_arm != null; robot_arm = robot_arm.NextMovePeer())
	{
		if ( !robot_arm || !robot_arm.IsValid() ) continue
		if ( robot_arm.GetClassname() == "tf_wearable_robot_arm") { robot_arm.Kill(); break; }
	}
}

//-----------------------------------------------------------------------------
// Purpose: Deletes weapons and all wearables tied to it.
//-----------------------------------------------------------------------------
::KillWepAll <- function(wep=null)
{
	if ( type( wep ) != "instance") {
		GTFW.DevPrint(1,"KillWepAll", "Function failed. Parameter 1 not an instance.");
		return null;
	}
	local player = wep.GetOwner();
	
	if( wep && wep.IsValid() )
	{
		local wearable = GetPropEntity(wep, "m_hExtraWearable");
		local wearable_vm = GetPropEntity(wep, "m_hExtraWearableViewModel");
		if ( wearable && wearable.IsValid() ) {
			GTFW.DevPrint(0,"KillWepAll", "Deleted "+wearable + " from " +GTFW.GetNetName(player));
			wearable.Kill();
		}
		if ( wearable_vm && wearable_vm.IsValid() ) {
			GTFW.DevPrint(0,"KillWepAll", "Deleted VM "+wearable + " from " +GTFW.GetNetName(player));
			wearable_vm.Kill();
		}
		
		wep.ValidateScriptScope()
		local wep_scope = wep.GetScriptScope();
		local prop;


/*		
//Commented out bc they're in wep_scope.GTFW_wearables_on_weapon
		if ( "viewModelProp" in wep_scope ) {
			prop = wep_scope.viewModelProp;
			if ( prop && prop.IsValid() )
				prop.Kill();
		}
		if ( "worldModelProp" in wep_scope ) {
			prop = wep_scope.worldModelProp;
			if ( prop && prop.IsValid() )
				prop.Kill();
		}
		if ( "worldModelPropWearable" in wep_scope ) {
			prop = wep_scope.worldModelPropWearable;
			if ( prop && prop.IsValid() )
				prop.Kill();
		}
*/		
		if ( "GTFW_wearables_on_weapon" in wep_scope ) {
			local wearable_on_weapon = wep_scope.GTFW_wearables_on_weapon;
			foreach ( k, wearable in wearable_on_weapon ) {
				
				if ( !wearable || !wearable.IsValid() ) continue
				
				wearable.Kill();
			}
		}

//from AddWeapon() under GiveWeapon().nut
// this is used for changing a weapon's slot.
		if ( "GTFW_parent_weapon" in wep_scope ) {
			prop = wep_scope.GTFW_parent_weapon;
			if ( prop && prop.IsValid() )
				prop.Kill();
		}
		
	//Destroy() fixes a glitch where killing the medigun doesn't stop healing patient
		if ( wep.GetClassname() == "tf_weapon_medigun" )
			wep.Destroy()
		else wep.Kill();
	}
}


//-----------------------------------------------------------------------------
// Purpose: For testing if weapon is custom
// If "bool" set to "true", adds bit flags to make the weapon custom
// If "bool" set to "false", removes the bits of a custom weapon
//-----------------------------------------------------------------------------
::IS_CUSTOM <- function(weapon, bool=null)
{
	weapon.ValidateScriptScope();
	local wepScope = weapon.GetScriptScope();
// Adds Custom bits if bool==true
	if ( bool ) {
		wepScope.IsCustom <- true;
		GTFW.DevPrint(0,"IS_CUSTOM", format("weapon %s's scope.IsCustom <- true",weapon.tostring()));
	}
// Removes Custom bits if bool==false
	else if ( bool == false ) {
		wepScope.IsCustom <- false;
		GTFW.DevPrint(0,"IS_CUSTOM", format("weapon %s's scope.IsCustom <- false",weapon.tostring()));
	}
	
	if ( "IsCustom" in wepScope ) return wepScope.IsCustom;
	else return false;
}


//-----------------------------------------------------------------------------
// Purpose: Finds Gunslinger on player, but only for Engineer. If found, returns true.
//-----------------------------------------------------------------------------
::CTFPlayer.HasGunslinger <- function(gunslinger=null)
{
	if ( this.GetPlayerClass() == TF_CLASS_ENGINEER )
	{
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = GetPropEntityArray(this, "m_hMyWeapons", i)
			
			if ( wep == null ) continue
			if ( wep.GetClassname() != "tf_weapon_robot_arm" ) continue
				return true
		}
	}
	return false
}




//-----------------------------------------------------------------------------
// Purpose: Finds item ID of weapon.
//-----------------------------------------------------------------------------
::GetItemID <- function(wep=null,clearbits=0)
{
//Error if param is wep is null
	if ( type(wep) != "instance" && type(wep) != "integer"  ) {
		GTFW.DevPrint(1,"GetItemID", "Function failed. Parameter 1 is not a handle or int. Returning null.");
		return wep;
	}
	
	local id = wep;
	if ( type(wep) == "instance"  )
		id = GetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex");
	
//clears custom bits if "ForceNotCustom" bit set
	if ( clearbits & ForceNotCustom ) {
		id = id & ( (1 << 16) - 1)
		//for ( local bit = 16; bit <= 21; bit++ ) {
		//	id = id & ~( 1 << bit );
		//}
	}

	return id
}


//-----------------------------------------------------------------------------
// Purpose: removes base item stats.
//-----------------------------------------------------------------------------
::GTFW.WipeAllAttributes <- function(wep=null)
{
	if ( type(wep) != "instance" ) {
		GTFW.DevPrint(1,"WipeItemStats", "Function failed. Parameter 1 is not a handle. Returning null.");
		return null;
	}
	
	SetPropInt(wep, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", 1);	// Removes all pre-existing stats of the weapon.
	return wep
}

	
//-----------------------------------------------------------------------------
// Purpose: gets player ID (for CreateCustomWearable())
//-----------------------------------------------------------------------------

::GTFWplyManager <- Entities.FindByClassname(null, "tf_player_manager");
::GTFW_GetPlayerUserID <- function(player)
{
    return GetPropIntArray(GTFWplyManager, "m_iUserID", player.entindex());
}


